@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
 ## A real Try Catch Block Implementation Using Perl Filter - ~/lib/Nice/Try.pm
-## Version v1.0.1
+## Version v1.0.2
 ## Copyright(c) 2021 DEGUEST Pte. Ltd.
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2020/05/17
-## Modified 2021/05/18
+## Modified 2021/05/19
 ## All rights reserved
 ## 
 ## This program is free software; you can redistribute  it  and/or  modify  it
@@ -23,7 +23,7 @@ BEGIN
     use Scalar::Util ();
     use List::Util ();
     use Want ();
-    our $VERSION = 'v1.0.1';
+    our $VERSION = 'v1.0.2';
     our $ERROR;
     our( $CATCH, $DIED, $EXCEPTION, $FINALLY, $HAS_CATCH, @RETVAL, $SENTINEL, $TRY, $WANTARRAY );
 }
@@ -34,7 +34,7 @@ our $SENTINEL = bless( {} => __PACKAGE__ . '::SENTINEL' );
 sub import
 {
     my( $this, @arguments ) = @_ ;
-    my $class = caller();
+    my $class = CORE::caller();
     my $hash = { @arguments };
     $hash->{debug} = 0 if( !CORE::exists( $hash->{debug} ) );
     $hash->{no_filter} = 0 if( !CORE::exists( $hash->{no_filter} ) );
@@ -188,8 +188,8 @@ sub _message
     return if( $self->{debug} < $level );
     my @data = @_;
     $stackFrame = 0;
-    my( $pkg, $file, $line, @otherInfo ) = caller( $stackFrame );
-    my $sub = ( caller( $stackFrame + 1 ) )[3];
+    my( $pkg, $file, $line, @otherInfo ) = CORE::caller( $stackFrame );
+    my $sub = ( CORE::caller( $stackFrame + 1 ) )[3];
     my $sub2 = substr( $sub, rindex( $sub, '::' ) + 2 );
     my $txt = "${pkg}::${sub2}( $self ) [$line]: " . join( '', map( ref( $_ ) eq 'CODE' ? $_->() : $_, @data ) );
     $txt    =~ s/\n$//gs;
@@ -205,8 +205,8 @@ sub _messagef
     my @data = @_;
     $stackFrame = 0;
     my $fmt = shift( @data );
-    my( $pkg, $file, $line, @otherInfo ) = caller( $stackFrame );
-    my $sub = ( caller( $stackFrame + 1 ) )[3];
+    my( $pkg, $file, $line, @otherInfo ) = CORE::caller( $stackFrame );
+    my $sub = ( CORE::caller( $stackFrame + 1 ) )[3];
     my $sub2 = substr( $sub, rindex( $sub, '::' ) + 2 );
     my $txt = "${pkg}::${sub2}( $self ) [$line]: " . sprintf( $fmt, map( ref( $_ ) eq 'CODE' ? $_->() : $_, @data ) );
     $txt    =~ s/\n$//gs;
@@ -621,12 +621,12 @@ EOT
             
             my $try_sub = <<EOT;
 CORE::local \$Nice::Try::WANT;
+CORE::local ( \$Nice::Try::EXCEPTION, \$Nice::Try::DIED, \@Nice::Try::RETVAL, \@Nice::Try::VOID );
 CORE::local \$Nice::Try::TRY = CORE::sub
 {
     CORE::do __TRY_OPEN_NL__{ __BLOCK_PLACEHOLDER__ };__TRY__CLOSE_NL__
-    CORE::return( \$Nice::Try::SENTINEL );
+    CORE::return( \$Nice::Try::VOID[0] = \$Nice::Try::SENTINEL );
 };
-CORE::local ( \$Nice::Try::EXCEPTION, \$Nice::Try::DIED, \@Nice::Try::RETVAL, \@Nice::Try::VOID );
 __FINALLY_BLOCK__ CORE::local \$Nice::Try::HAS_CATCH = $has_catch_clause;
 CORE::local \$Nice::Try::WANTARRAY = CORE::wantarray;
 EOT
@@ -703,6 +703,7 @@ EOT
             elsif( \$Nice::Try::WANT eq 'BOOLEAN' )
             {
                 \$Nice::Try::RETVAL[0] = &\$Nice::Try::TRY ? 1 : 0;
+                \$Nice::Try::RETVAL[0] = \$Nice::Try::VOID[0] if( scalar( \@Nice::Try::VOID ) );
             }
             elsif( \$Nice::Try::WANT eq 'VOID' )
             {
@@ -889,7 +890,8 @@ EOT
                 }
                 elsif( \$Nice::Try::WANT eq 'BOOLEAN' )
                 {
-                    \$Nice::Try::RETVAL[0] = \&\$Nice::Try::CATCH ? 1 : 0;
+                    my \$this = \&\$Nice::Try::CATCH ? 1 : 0;
+                    \$Nice::Try::RETVAL[0] = \$Nice::Try::VOID[0] if( scalar( \@Nice::Try::VOID ) );
                 }
                 elsif( \$Nice::Try::WANT eq 'VOID' )
                 {
@@ -1209,9 +1211,10 @@ sub _process_caller
     return( $elem ) if( !$elem->children );
     foreach my $e ( $elem->elements )
     {
-        $self->_messagef( 6, "Checking element: [%d] class %s with %d children and value '%s'\n", $e->line_number, $e->class, ( $e->can('elements') ? scalar( $e->elements ) : 0 ), $e->content );
+        my $content = $e->content // '';
+        $self->_messagef( 6, "Checking element: [%d] class %s with %d children and value '%s'\n", $e->line_number, $e->class, ( $e->can('elements') ? scalar( $e->elements ) : 0 ), $content );
         my $class = $e->class;
-        if( $class eq 'PPI::Token::Word' && $e->content =~ /^(?:CORE\::)?(?:GLOBAL\::)?caller$/ )
+        if( $class eq 'PPI::Token::Word' && $content =~ /^(?:CORE\::)?(?:GLOBAL\::)?caller$/ )
         {
             $self->_message( 4, "Found caller, replacing with ", 'Nice::Try::caller_' . $where );
             $e->set_content( 'Nice::Try::caller_' . $where );
@@ -1235,16 +1238,19 @@ sub _process_loop_breaks
     $self->_messagef( 5, "Checking %d elements for '$elem'", scalar( $elem->elements ) );
     foreach my $e ( $elem->elements )
     {
-        $self->_messagef( 6, "Checking element: [%d] class %s with %d children and value '%s'\n", $e->line_number, $e->class, ( $e->can('elements') ? scalar( $e->elements ) : 0 ), $e->content );
+        my $content = $e->content // '';
+        $self->_messagef( 6, "Checking element: [%d] class %s with %d children and value '%s'\n", $e->line_number, $e->class, ( $e->can('elements') ? scalar( $e->elements ) : 0 ), $content );
         my $class = $e->class;
         # We found a for, foreach or while loops and we skip, because if there are any break words (next, last, redo) inside, it is not our problem.
         if( $class eq 'PPI::Structure::For' ||
-            ( $class eq 'PPI::Statement::Compound' && $e->first_element->content =~ /^(foreach|while)$/ ) )
+            ( $class eq 'PPI::Statement::Compound' && 
+              CORE::defined( $e->first_element->content ) && 
+              $e->first_element->content =~ /^(foreach|while)$/ ) )
         {
             # $self->_message( 6, "Skipping it. Its first word was '", $e->first_element->content, "'" );
             next;
         }
-        elsif( $class eq 'PPI::Token::Word' && $e->content =~ /^(?:CORE\::)?(?:GLOBAL\::)?(next|last|redo)$/ )
+        elsif( $class eq 'PPI::Token::Word' && $content =~ /^(?:CORE\::)?(?:GLOBAL\::)?(next|last|redo)$/ )
         {
             $e->set_content( qq{CORE::return( '__} . uc( $1 ) . qq{__' )} );
         }
@@ -1252,22 +1258,24 @@ sub _process_loop_breaks
         {
             my $words = $e->find( 'PPI::Token::Word' );
             $self->_messagef( 5, "Found %d word elements inside break element.", scalar( @$words ) );
-            $self->_message( 5, "Word 1 -> ", $words->[0]->content );
-            $self->_message( 5, "Word 2 -> ", $words->[1]->content ) if( scalar( @$words ) > 1 );
+            my $word1 = ( scalar( @$words ) ? $words->[0]->content // '' : '' );
+            my $word2 = ( scalar( @$words ) > 1 ? $words->[1]->content // '' : '' );
+            $self->_message( 5, "Word 1 -> ", $word1 );
+            $self->_message( 5, "Word 2 -> ", $word2 ) if( scalar( @$words ) > 1 );
             # $self->_browse( $e );
             # If we found a break word without a label, i.e. next, last, redo, 
             # we replace it with a special return statement
             if( ( scalar( @$words ) == 1 ||
-                  ( scalar( @$words ) > 1 && $words->[1]->content =~ /^(for|foreach|given|if|unless|until|while)$/ )
+                  ( scalar( @$words ) > 1 && $word2 =~ /^(for|foreach|given|if|unless|until|while)$/ )
                 ) && 
                 (
-                  $words->[0]->content eq 'next' ||
-                  $words->[0]->content eq 'last' ||
-                  $words->[0]->content eq 'redo' ) )
+                  $word1 eq 'next' ||
+                  $word1 eq 'last' ||
+                  $word1 eq 'redo' ) )
             {
                 # We add our special return value. Notice that we use 'return' and not 
                 # 'CORE::return'. See below why.
-                my $break_code = qq{return( '__} . uc( $words->[0]->content ) . qq{__' )};
+                my $break_code = qq{return( '__} . uc( $word1 ) . qq{__' )};
                 # e.g. next if( $i == 2 );
                 # next and if are both treated as 'word' by PPI
                 if( scalar( @$words ) > 1 )
@@ -1632,7 +1640,7 @@ Also since version 1.0.0, L<Nice::Try> is context aware:
 
 =head1 VERSION
 
-    v1.0.1
+    v1.0.2
 
 =head1 DESCRIPTION
 
@@ -2254,7 +2262,7 @@ Other cases are:
 
 See L<Want> for more information on how you can benefit from it.
 
-Currently lvalues are no implemented and will be in future releases. Also note that L<Want> does not work with tied object and overloaded classes. L<Nice::Try> detects this and disable automatically support for L<Want> if the object is overloaded.
+Currently lvalues are no implemented and will be in future releases. Also note that L<Want> does not work within tie-handlers. It would trigger a segmentation fault. L<Nice::Try> detects this and disable automatically support for L<Want> if used inside a tie-handler, reverting to regular L<perlfunc/wantarray> context.
 
 Also, for this rich context awareness to be used, obviously try-catch would need to be inside a subroutine, otherwise there is no rich context other than the one the regular L<perlfunc/wantarray> provides.
 
